interface RoomId {
    roomId: string;
}
interface PayloadCommon extends RoomId {
    sender: string;
    receiver: string;
}
interface PayloadOffer extends PayloadCommon {
    offer: RTCSessionDescriptionInit;
}
interface PayloadAnswer extends PayloadCommon {
    answer: RTCSessionDescriptionInit;
}
interface PayloadCandidate extends PayloadCommon {
    candidate: RTCIceCandidate;
}
declare enum Topic {
    OfferSend = "mche:offer:send",
    AnswerSend = "mche:answer:send",
    CandidateSend = "mche:candidate:send"
}
interface Payload {
    [Topic.OfferSend]: PayloadOffer;
    [Topic.AnswerSend]: PayloadAnswer;
    [Topic.CandidateSend]: PayloadCandidate;
    open: {
        data: {
            id: string;
            userData?: Record<string, unknown>;
        }[];
    } & RoomId;
    close: {
        data: {
            id: string;
        };
    } & RoomId;
}

type TypedArray = Int8Array | Uint8Array | Int16Array | Uint16Array | Int32Array | Uint32Array | Float32Array | Float64Array | BigInt64Array | BigUint64Array;
type RTCChannelData = string | Blob | ArrayBuffer | DataView | TypedArray;

interface MCHelperOptions {
    /**
     * The URL of the signaling server.
     * @example "wss://example.com:8080"
     */
    signalingServerUrl: string;
    /**
     * The ID of the peer.
     * If not set, will be generated using `nanoid(6)`.
     * @default nanoid(6)
     */
    id?: string;
    /**
     * Whether to print debug information.
     * - `'verbose'`: print all debug information
     * - `true`: print connection information
     *
     * @default false
     */
    debug?: boolean | 'verbose';
    /**
     * ICE servers.
     * @default
     * [
     *  { urls: 'stun:stun.l.google.com:19302', }
     * ]
     */
    iceServers?: RTCIceServer[];
    /**
     * Room id.
     *
     * Same room id will be able to communicate with each other.
     */
    roomId: string;
}
type OnJoinCallback = (data: Payload['open']['data']) => void;
type OnLeaveCallback = (data: Payload['close']['data']) => void;
declare class MCHelper<B> {
    #private;
    constructor(options: MCHelperOptions);
    get id(): string;
    serializeObjectPayload(payload: B): string;
    deserializeObjectPayload(payload: RTCChannelData): B;
    /**
     * If you want to broadcast messages to some peers, you can use `send` function
     * @param message RTCChannelData, you can use `serializeObjectPayload` to serialize object to string.
     * https://developer.mozilla.org/en-US/docs/Web/API/RTCDataChannel/send#data
     */
    broadcast(message: RTCChannelData): void;
    /**
     * If you want to broadcast messages to all peers, you can use `broadcast` function
     * @param targetIds The ID of the peers.
     * @param message RTCChannelData, you can use `serializeObjectPayload` to serialize object to string.
     * https://developer.mozilla.org/en-US/docs/Web/API/RTCDataChannel/send#data
     */
    send(targetIds: string[], message: RTCChannelData): void;
    onMessageChannelReady(callback: (channel: RTCDataChannel) => void): void;
    onJoin(callback: OnJoinCallback): void;
    onLeave(callback: OnLeaveCallback): void;
    /**
     * @returns cleanup fn
     */
    onBroadcast<D extends RTCChannelData>(callback: (message: MessageEvent<D>) => void): () => void;
    close(): void;
    get signallingServerWS(): {
        ws: {
            readonly ws: WebSocket | null;
            open(): void;
            close(): void;
            send(event: string, data: any): void;
            subscribe(event: string, callback: (data?: any) => void, once?: boolean | undefined): () => void;
        };
        registerCallbacks: (callbacks: Partial<{
            onConnected: (ws: WebSocket, event: Event) => void;
            onDisconnected: (ws: WebSocket, event: Event) => void;
            onError: (ws: WebSocket, event: Event) => void;
        }>) => void;
    };
}

export { MCHelper, type MCHelperOptions, type OnJoinCallback, type OnLeaveCallback, type RTCChannelData, type TypedArray };
