// src/index.ts
import { nanoid } from "nanoid/non-secure";

// src/utils.ts
function error(error2) {
  console.error(`[MCHE Error] ${error2.message}`);
  console.error(error2.stack);
}
function log(...args) {
  console.log("[MCHE Log]", ...args);
}

// src/error-handling.ts
var sctpCauseCodes = [
  "No SCTP error",
  "Invalid stream identifier",
  "Missing mandatory parameter",
  "Stale cookie error",
  "Sender is out of resource (i.e., memory)",
  "Unable to resolve address",
  "Unrecognized SCTP chunk type received",
  "Invalid mandatory parameter",
  "Unrecognized parameters",
  "No user data (SCTP DATA chunk has no data)",
  "Cookie received while shutting down",
  "Restart of an association with new addresses",
  "User-initiated abort",
  "Protocol violation"
];
function handleDCError(ev, connection) {
  const err = ev.error;
  console.error("WebRTC error: ", err.message);
  switch (err.errorDetail) {
    case "sdp-syntax-error":
      console.error("    SDP syntax error in line ", err.sdpLineNumber);
      break;
    case "sctp-failure": {
      if (err.sctpCauseCode && err.sctpCauseCode < sctpCauseCodes.length)
        console.error("    SCTP failure: ", err.sctpCauseCode);
      else
        console.error("    Unknown SCTP error");
      break;
    }
    case "dtls-failure":
      if (err.receivedAlert)
        console.error("    Received DLTS failure alert: ", err.receivedAlert);
      if (err.sentAlert)
        console.error("    Sent DLTS failure alert: ", err.receivedAlert);
      break;
  }
  connection.close();
}

// src/connection.ts
var ICE_SERVERS = [
  {
    urls: "stun:stun.l.google.com:19302"
  }
];
var MCHEConnection = class {
  #peerConnection;
  #datachannel = null;
  #datachannelCallbacks = [];
  #datachannelReadyCallbacks = [];
  #debug;
  constructor(onDataChannelMessage, debug, customIceServers) {
    this.#peerConnection = new RTCPeerConnection({
      iceServers: [
        ...customIceServers ?? [],
        ...ICE_SERVERS
      ]
    });
    this.#peerConnection.ondatachannel = (event) => {
      const channel = this.#datachannel = event.channel;
      if (this.#debug)
        log("RemoteDataChannel created.", channel);
      this.#datachannelCallbacks.forEach((callback) => {
        callback(channel);
      });
    };
    this.#datachannelCallbacks.push((channel) => {
      channel.onmessage = (event) => {
        onDataChannelMessage(event);
      };
      channel.onopen = () => {
        this.#datachannelReadyCallbacks.forEach((callback) => {
          callback(channel);
        });
      };
      channel.onerror = (event) => {
        handleDCError(event, this);
      };
    });
    this.#debug = debug;
  }
  async createOffer() {
    const channel = this.#datachannel = this.#peerConnection.createDataChannel("datachannel");
    if (this.#debug)
      log("LocalDataChannel created.", this.#datachannel);
    this.#datachannelCallbacks.forEach((callback) => {
      callback(channel);
    });
    const offer = await this.#peerConnection.createOffer();
    await this.#peerConnection.setLocalDescription(offer);
    return offer;
  }
  async receiveOffer(offer) {
    await this.#peerConnection.setRemoteDescription(offer);
  }
  async createAnswer() {
    const answer = await this.#peerConnection.createAnswer();
    await this.#peerConnection.setLocalDescription(answer);
    return answer;
  }
  async receiveAnswer(answer) {
    await this.#peerConnection.setRemoteDescription(answer);
  }
  async addIceCandidate(candidate) {
    await this.#peerConnection.addIceCandidate(candidate);
  }
  async createDataChannel(label) {
    const channel = this.#peerConnection.createDataChannel(label);
    return channel;
  }
  onCandidate(callback) {
    this.#peerConnection.onicecandidate = (event) => {
      if (event.candidate)
        callback(event.candidate);
    };
  }
  addDataChannelCallback(callback) {
    if (this.#datachannel) {
      callback(this.#datachannel);
      return;
    }
    this.#datachannelCallbacks.push(callback);
  }
  addDataChannelReadyCallback(callback) {
    if (this.#datachannel?.readyState === "open") {
      callback(this.#datachannel);
      return;
    }
    this.#datachannelReadyCallbacks.push(callback);
  }
  close() {
    this.#peerConnection.close();
    this.#datachannel?.close();
    this.#datachannelCallbacks.length = 0;
  }
};

// src/signaling.ts
import { createWSHE } from "wshe";
var SignalingServerClient = class {
  #ws;
  #roomId;
  #onConnectedCallbacks = [];
  #onDisconnectedCallbacks = [];
  #onErrorCallbacks = [];
  constructor(url, roomId) {
    this.#ws = createWSHE(url, {
      immediate: true,
      onConnected: (ws, event) => {
        this.#onConnectedCallbacks.forEach((callback) => callback(ws, event));
      },
      onDisconnected: (ws, event) => {
        this.#onDisconnectedCallbacks.forEach((callback) => callback(ws, event));
      },
      onError: (ws, event) => {
        this.#onErrorCallbacks.forEach((callback) => callback(ws, event));
      }
    });
    this.#roomId = roomId;
  }
  send(topic, data) {
    this.#ws.send(topic, {
      ...data,
      roomId: this.#roomId
    });
  }
  on(topic, callback) {
    this.#ws.subscribe(topic, (data) => {
      if (data.roomId !== this.#roomId)
        return;
      callback(data);
    });
  }
  close() {
    this.#onConnectedCallbacks.length = 0;
    this.#onDisconnectedCallbacks.length = 0;
    this.#onErrorCallbacks.length = 0;
    this.#ws.close();
  }
  get ws() {
    return {
      ws: this.#ws,
      registerCallbacks: (callbacks) => {
        if (callbacks.onConnected)
          this.#onConnectedCallbacks.push(callbacks.onConnected);
        if (callbacks.onDisconnected)
          this.#onDisconnectedCallbacks.push(callbacks.onDisconnected);
        if (callbacks.onError)
          this.#onErrorCallbacks.push(callbacks.onError);
      }
    };
  }
};

// src/index.ts
var randomId = () => nanoid(6);
var MCHelper = class {
  #options;
  #id;
  #connectionPool = /* @__PURE__ */ new Map();
  #signallingServer;
  #dataBuffer = [];
  #dataBufferCallbacks = [];
  #onJoinCallbacks = [];
  #onLeaveCallbacks = [];
  #onMessageChannelReadyCallbacks = [];
  constructor(options) {
    this.#options = options;
    this.#id = options.id || randomId();
    this.#signallingServer = new SignalingServerClient(options.signalingServerUrl, options.roomId);
    this.#bindEvents().catch(error);
    this.#bindSignalingEvents();
    this.#registerOnSignalingServer();
  }
  #bindSignalingEvents() {
    this.#signallingServer.on("open", ({ data }) => {
      if (this.#options.debug)
        log("Open connection with", data);
      data.forEach(({ id }) => {
        this.#registerRemote(id);
      });
      this.#onJoinCallbacks.forEach((callback) => {
        callback(data);
      });
    });
    this.#signallingServer.on("close", ({ data }) => {
      const { id } = data;
      const connection = this.#connectionPool.get(id);
      if (connection) {
        if (this.#options.debug)
          log("Close connection with", id);
        connection.close();
        this.#connectionPool.delete(id);
        this.#onLeaveCallbacks.forEach((callback) => {
          callback(data);
        });
      }
    });
  }
  #addMessageToBuffer(message) {
    this.#dataBuffer.push(message);
    this.#dataBufferCallbacks.forEach((callback) => {
      callback(message);
      this.#dataBuffer.length = 0;
    });
  }
  #getConnection() {
    return new MCHEConnection((message) => {
      this.#addMessageToBuffer(message);
    }, this.#options.debug, this.#options.iceServers);
  }
  #addNewConnection(id, conn) {
    this.#connectionPool.set(id, conn);
    this.#onMessageChannelReadyCallbacks.forEach((callback) => {
      conn.addDataChannelReadyCallback(callback);
    });
  }
  #registerOnSignalingServer() {
    const ws = this.#signallingServer.ws;
    ws.registerCallbacks({
      onConnected: () => {
        ws.ws.send("register", JSON.stringify({
          roomId: this.#options.roomId,
          userId: this.#options.id
        }));
      }
    });
  }
  async #bindEvents() {
    this.#signallingServer.on("mche:offer:send" /* OfferSend */, async ({ receiver, offer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received offer from", sender, "payload is", offer);
      let connection = this.#connectionPool.get(sender);
      if (!connection) {
        connection = this.#getConnection();
        this.#addNewConnection(sender, connection);
      }
      await connection.receiveOffer(offer);
      const answer = await connection.createAnswer();
      const payload = {
        answer,
        // swap sender and receiver
        receiver: sender,
        sender: receiver
      };
      if (this.#options.debug)
        log("Sending answer to", sender, "payload is", payload);
      this.#signallingServer.send("mche:answer:send" /* AnswerSend */, payload);
    });
    this.#signallingServer.on("mche:answer:send" /* AnswerSend */, async ({ receiver, answer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received answer from", sender, "payload is", answer);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.receiveAnswer(answer);
    });
    this.#signallingServer.on("mche:candidate:send" /* CandidateSend */, async ({ receiver, candidate, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received candidate from", sender, "payload is", candidate);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.addIceCandidate(candidate);
    });
  }
  /**
   * Call this function to connection other peers.
   * @returns The ID of the peer. You need **hold** this ID to do next operations.
   */
  async #registerRemote(remoteId) {
    const nonNullRemoteId = remoteId || randomId();
    const connection = this.#getConnection();
    connection.onCandidate((candidate) => {
      const payload = {
        receiver: nonNullRemoteId,
        sender: this.#id,
        candidate
      };
      if (this.#options.debug)
        log("Sending candidate to", nonNullRemoteId, "payload is", payload);
      this.#signallingServer.send("mche:candidate:send" /* CandidateSend */, payload);
    });
    this.#addNewConnection(nonNullRemoteId, connection);
    await this.#connect(nonNullRemoteId);
    return nonNullRemoteId;
  }
  async #connect(peerId) {
    const connection = this.#connectionPool.get(peerId);
    if (!connection) {
      error(new Error(`Peer ${peerId} not found.`));
      return;
    }
    const offer = await connection.createOffer();
    const payload = {
      offer,
      sender: this.#id,
      receiver: peerId
    };
    if (this.#options.debug)
      log("Sending offer to", peerId, "payload is", payload);
    this.#signallingServer.send("mche:offer:send" /* OfferSend */, payload);
  }
  // ----------------- Public API -----------------
  get id() {
    return this.#id;
  }
  serializeObjectPayload(payload) {
    return JSON.stringify(payload);
  }
  deserializeObjectPayload(payload) {
    return JSON.parse(payload);
  }
  /**
   * If you want to broadcast messages to some peers, you can use `send` function
   * @param message RTCChannelData, you can use `serializeObjectPayload` to serialize object to string.
   * https://developer.mozilla.org/en-US/docs/Web/API/RTCDataChannel/send#data
   */
  broadcast(message) {
    const payload = {
      sender: this.#id,
      receiver: "broadcast",
      message
    };
    if (this.#options.debug === "verbose")
      log("Broadcasting message", message, "payload is", payload);
    this.#connectionPool.forEach((connection) => {
      connection.addDataChannelReadyCallback((channel) => {
        channel.send(payload.message);
      });
    });
  }
  /**
   * If you want to broadcast messages to all peers, you can use `broadcast` function
   * @param targetIds The ID of the peers.
   * @param message RTCChannelData, you can use `serializeObjectPayload` to serialize object to string.
   * https://developer.mozilla.org/en-US/docs/Web/API/RTCDataChannel/send#data
   */
  send(targetIds, message) {
    const payload = {
      sender: this.#id,
      receiver: targetIds.toString(),
      message
    };
    const targetIdsSet = new Set(targetIds);
    if (this.#options.debug === "verbose")
      log("Sending message", message, "payload is", payload);
    this.#connectionPool.forEach((connection, id) => {
      if (targetIdsSet.has(id)) {
        connection.addDataChannelReadyCallback((channel) => {
          channel.send(payload.message);
        });
      }
    });
  }
  onMessageChannelReady(callback) {
    this.#onMessageChannelReadyCallbacks.push(callback);
  }
  onJoin(callback) {
    this.#onJoinCallbacks.push(callback);
  }
  onLeave(callback) {
    this.#onLeaveCallbacks.push(callback);
  }
  /**
   * @returns cleanup fn
   */
  onBroadcast(callback) {
    const fn = (message) => {
      if (this.#options.debug === "verbose")
        log("Get broadcasted message", message);
      callback(message);
    };
    this.#dataBufferCallbacks.push(fn);
    return () => {
      const index = this.#dataBufferCallbacks.indexOf(fn);
      if (index !== -1)
        this.#dataBufferCallbacks.splice(index, 1);
    };
  }
  close() {
    this.#connectionPool.forEach((connection) => {
      connection.close();
    });
    this.#signallingServer.close();
    this.#onJoinCallbacks.length = 0;
    this.#onLeaveCallbacks.length = 0;
  }
  get signallingServerWS() {
    return this.#signallingServer.ws;
  }
};
export {
  MCHelper
};
